import time
from vispy import scene
from vispy.scene import Text
from SpectrogramData import SpectrogramData

class ScanDisplay:
    def __init__(self, ESWData:SpectrogramData):
        self.ESWData = ESWData
        self.graph_added = False
        self.graph_title = "ESW Receiver Scan"
        self.Xax_title = "Frequency [MHz]"
        self.Yax_title = "Level [dBμV]"
        self.graph_colors = {"background":"#000000", "text":"#6b6c6e", "line1":"#E6D100", "line2":"#00E616", "line3":"#E68E00", "line4":"#E62EE5", "line5":"#BFBFBF", "line6":"#255CF2"}
        self.x_lim = ESWData.x_range
        self.y_lim = ESWData.y_range
        self.idx = len(ESWData.detectors)
        self.plot_lines = []
        for x in range(self.idx):
            self.plot_lines.append([])
        self.instrumentTimeDifference = 0

        self.canvas = scene.SceneCanvas(title='R&S ESW Spectrogram', keys='interactive', bgcolor=self.graph_colors["background"], size = self.ESWData.graph_size)
        self.grid = self.canvas.central_widget.add_grid(spacing=0)
        self.title = scene.Label(self.graph_title, color=self.graph_colors["text"])
        self.textDelay = Text('Graph delay: 0.00 s', parent=self.canvas.scene, color='red')
        self.textDelay.font_size = 16
        self.textDelay.pos = 250, 70
        self.title.height_max = 40
        self.grid.add_widget(self.title, row=0, col=2, col_span=1)
        right_padding = self.grid.add_widget(row=0, col=3, row_span=2, col_span = 1)
        right_padding.width_max = 50
        left_padding = self.grid.add_widget(row=0, col=0, row_span=2, col_span = 1)
        left_padding.width_max = 20
        self.y_axis = scene.AxisWidget(orientation='left',
                         axis_label=self.Yax_title,
                         axis_font_size=12,
                         axis_label_margin=50,
                         tick_label_margin=5,
                         axis_color = (0.6, 0.6, 0.6),
                         tick_color = (0.5, 0.5, 0.5),
                         text_color = self.graph_colors["text"],
                         domain = (self.y_lim[0], self.y_lim[1])
                         )
        self.y_axis.width_max = 80
        self.grid.add_widget(self.y_axis, row=1, col=1)
        self.x_axis = scene.AxisWidget(orientation='bottom',
                         axis_label=self.Xax_title,
                         axis_font_size=12,
                         axis_label_margin=50,
                         tick_label_margin=20,
                         axis_color = (0.6, 0.6, 0.6),
                         tick_color = (0.5, 0.5, 0.5),
                         text_color = self.graph_colors["text"])
        self.x_axis.height_max = 80
        self.grid.add_widget(self.x_axis, row=2, col=2)
        self.viewbox = self.grid.add_view(row=1, col=2, camera='panzoom', border_color = 'black')
        self.viewbox.camera.set_range((self.x_lim[0], self.x_lim[1]), (self.y_lim[0], self.y_lim[1]))
        self.gridlines = scene.visuals.GridLines(color=self.graph_colors["text"])
        self.viewbox.add(self.gridlines)
        self.y_axis.link_view(self.viewbox)
        self.x_axis.link_view(self.viewbox)
        self.canvas.show()

    def AddGraphPlot(self):
        if not self.graph_added:
            for i in range(self.idx):
                self.plot_lines[i] = scene.Line([], self.graph_colors['line'+str(i+1)], parent=self.viewbox.scene)
            self.graph_added = True

    def Update(self):
        """
        This function updates the graph plot with the next available frame. If frames come in faster they
        are plotted, this will be made visible by the shown graph delay. An alternative is to always plot
        the latest frame to avoid delays, but this defeats the real-time purpose as not all frames are shown.
        :return: None
        """
        while True:
            g = self.ESWData.get_data_single()
            if g == 1:
                self.textDelay.text = 'Graph delay: '+str("{:.2f}".format(self.ESWData.dataDelay))+" s"
                self.AddGraphPlot()
                for i in range(self.idx):
                    self.plot_lines[i].set_data(pos=self.ESWData.data_recorder[(0,i+1),:].T)
            elif g == 2:
                self.textDelay.text = 'Waiting for Trigger'
            time.sleep(0.01)
