import time
from vispy import scene
from vispy.visuals.transforms import STTransform
from vispy import color
import numpy as np
from SpectrogramData import SpectrogramData

class SpectrogramDisplay:
    def __init__(self, ESWData:SpectrogramData):
        self.ESWData = ESWData
        self.ntraces = 300
        self.graph_title = "ESW Spectrogram Plot ("+str(self.ESWData.y_range[0])+" dBμV - "+str(self.ESWData.y_range[1])+" dBμV)"
        self.Xax_title = "Frequency [MHz]"
        self.Yax_title = "Time [s]"
        self.graph_colors = {"background":"#000000", "text":"#FFFFFF", "line1":"#E6D100", "line2":"#00E616", "line3":"#E68E00", "line4":"#E62EE5", "line5":"#BFBFBF", "line6":"#255CF2"}

        self.canvas = scene.SceneCanvas(title='R&S ESW Spectrogram', keys='interactive', bgcolor=self.graph_colors["background"], size=self.ESWData.graph_size)

        self.grid = self.canvas.central_widget.add_grid(spacing=0)
        self.title = scene.Label(self.graph_title, color=self.graph_colors["text"])

        self.title.height_max = 40
        self.grid.add_widget(self.title, row=0, col=2, col_span=1)
        right_padding = self.grid.add_widget(row=0, col=3, row_span=2, col_span = 1)
        right_padding.width_max = 50
        left_padding = self.grid.add_widget(row=0, col=0, row_span=2, col_span = 1)
        left_padding.width_max = 20
        self.y_axis = scene.AxisWidget(orientation='left',
                         axis_label=self.Yax_title,
                         axis_font_size=12,
                         axis_label_margin=50,
                         tick_label_margin=5,
                         axis_color = (0.6, 0.6, 0.6),
                         tick_color = (0.5, 0.5, 0.5),
                         text_color = self.graph_colors["text"],
                         )
        self.y_axis.width_max = 80
        self.grid.add_widget(self.y_axis, row=1, col=1)
        self.x_axis = scene.AxisWidget(orientation='bottom',
                         axis_label=self.Xax_title,
                         axis_font_size=12,
                         axis_label_margin=50,
                         tick_label_margin=20,
                         axis_color = (0.6, 0.6, 0.6),
                         tick_color = (0.5, 0.5, 0.5),
                         text_color = self.graph_colors["text"],
                         )
        self.x_axis.height_max = 80
        self.grid.add_widget(self.x_axis, row=2, col=2)
        self.viewbox = self.grid.add_view(row=1, col=2, camera='panzoom', border_color = 'black')
        self.viewbox.camera.set_range((0, 1000), (-self.ntraces*ESWData.settings["Time"], 0))
        self.gridlines = scene.visuals.GridLines(color=self.graph_colors["text"])
        self.viewbox.add(self.gridlines)
        self.y_axis.link_view(self.viewbox)
        self.x_axis.link_view(self.viewbox)
        self.img_array = np.zeros((self.ntraces, ESWData.points), dtype='float32')
        self.image = scene.visuals.Image(self.img_array, interpolation='nearest', parent=self.viewbox.scene, method='auto', clim=self.ESWData.y_range, cmap=color.colormap.MatplotlibColormap('turbo'))
        st = STTransform(scale=((ESWData.settings["Stop"]-ESWData.settings["Start"])/ESWData.points, -ESWData.settings["Time"]), translate=(ESWData.settings["Start"]))
        self.image.transform = st

        self.canvas.show()

    def AddGraphPlot(self):
        return None

    def Update(self):
        """
        This function updates the graph plot with the next available frame. If frames come in faster they
        are plotted, this will be made visible by the shown graph delay. An alternative is to always plot
        the latest frame to avoid delays, but this defeats the real-time purpose as not all frames are shown.
        :return: None
        """
        while True:
            if self.ESWData.get_data_single() == 1:
                self.img_array = np.roll(self.img_array, 1, axis=0)
                self.img_array[0,:] = self.ESWData.data_recorder[1, 1-1::1]
                self.image.set_data(self.img_array)
                self.canvas.update()
            time.sleep(0.01)
